/**
 * Node module for downloads.
 * This changes the download behaviour.
 */

/* Private */
/* If no download settings are present, create defaults */
if (!global.settings.downloads) {
	global.settings.downloads = {
		'openOnCompleted': true
	}
}
/* Variables */
var _dlSettings = global.settings.downloads;
var _openByDialogIds = [];
var _popupWindow;
var _realDownloadIDs = {};

/* Get the current chrome setting for download.prompt_for_download */
var _showSaveAs;
chrome.settingsPrivate.getPref("download.prompt_for_download", function(pref){
	_showSaveAs = pref.value;
});

/**
 * function to set the chrome setting download.prompt_for_download
 * @param {Bool} show if true show the saveas dialog, if false don't, defaults to false
 */
function _setShowSaveAs(show = false) {
	chrome.settingsPrivate.setPref("download.prompt_for_download",
		show, '', function(res){console.log("result:" + res)});
	_showSaveAs = show;
}

/* Get the current chrome setting for download.default_directory */
var _defaultLocation;
chrome.settingsPrivate.getPref("download.default_directory", function(pref){
	_defaultLocation = pref.value;
});

/**
 * function to set the chrome setting download.prompt_for_download
 * @param {Bool} show if true show the saveas dialog, if false don't, defaults to false
 */
function _setDefaultLocation(path, cb = function(){}) {
	chrome.settingsPrivate.setPref("download.default_directory",
		path, '', cb);
	_defaultLocation = path;
}

function _getTemporaryDownloadLocation() {
	var os = require('os');
	return _dlSettings.temporaryDownloadLocation || os.tmpdir();
}

/**
 * Listener for completed downloads
 * If a download has completed and _dlSettings.openOnCompleted is true,
 * the downloaded file is opened by the OS default application for it.
 */
chrome.downloads.onChanged.addListener(function(dl){
	var openByDialog = _openByDialogIds.indexOf(dl.id) > -1;
	if (dl.state && dl.state.current === 'complete' &&
		(_dlSettings.openOnCompleted || _dlSettings.downloadBehavior === 'open' || openByDialog)) {
		_openByDialogIds.splice(_openByDialogIds.indexOf(dl.id), 1);
		chrome.downloads.search({id: dl.id}, function(dla) {
			nw.Shell.openItem(dla[0].filename);
		});
	}
});

/**
 * Listener for downloads, when their filename is determined
 * Depending on the settings a dilaog is injected into the webapp window,
 * asking the user what to do with a download.
 */
chrome.downloads.onDeterminingFilename.addListener(function(item, _suggest) {
	if (_realDownloadIDs[item.id] === undefined) {
		chrome.downloads.cancel(item.id);
		chrome.downloads.erase({id: item.id});
	} else {
		var suggestOptions = { filename: item.filename };
		if (_realDownloadIDs[item.id] === 'open') {
			suggestOptions['conflictAction'] = 'overwrite';
		}
		_suggest(suggestOptions);
		return;
	}
	if(_dlSettings.downloadBehavior === 'ask') {
		var path = require('path');
		var ModalDialog = require(path.join(process.cwd(), 'app', 'modules', 'modalDialogs.js'));
		var dlDialog = new ModalDialog(
			{
				'type': 'downloadDialog',
				'details': item.filename
			},
			global.DA.winWebApp.window.document.body,
			global.DA.winWebApp.window,
			function(){
				var ret = JSON.parse(dlDialog.getReturnValue());
				if (ret.action === 'open') {
					var currentShowSaveAs = _showSaveAs;
					var currentDefaultLocation = _defaultLocation;
					_setShowSaveAs(false);
					_setDefaultLocation(_getTemporaryDownloadLocation(), function(result) {
						chrome.downloads.download(
							{ url: item.url, conflictAction: 'overwrite' },
							function(id) {
								_realDownloadIDs[id] = ret.action;
								_openByDialogIds.push(id);
								setTimeout(function() {
									_setDefaultLocation(currentDefaultLocation);
								},100);
						});
					});
				} else {
					var currentShowSaveAs = _showSaveAs;
					if (ret.action === 'SAVE') {
						_setShowSaveAs(false);
					} else if (ret.action === 'SAVEAS') {
						_setShowSaveAs(true);
					}
					setTimeout(function() {
						chrome.downloads.download(
							{ url: item.url },
							function(id) {
								_realDownloadIDs[id] = ret.action;
						});
					}, 100);
					setTimeout(function() {
						_setShowSaveAs(currentShowSaveAs);
					}, 500);
				}
			},
			function(){
				chrome.downloads.cancel(item.id);
			}
		);
	} else {
		var currentShowSaveAs = _showSaveAs;
		var currentDefaultLocation = _defaultLocation;
		_setShowSaveAs(false);
		_setDefaultLocation(_getTemporaryDownloadLocation(), function(result) {
			chrome.downloads.download(
			{ url: item.url },
			function(id) {
				_realDownloadIDs[id] = 'open';
				_openByDialogIds.push(id);
				setTimeout(function() {
					_setDefaultLocation(currentDefaultLocation);
				},100);
			});
		});
	}
	return true;
});

/**
 * Open the download history window.
 */
function _openHistory() {
	var winDownloadHistoryOptions = {
		resizable: false,
		id: 'dlDownloadHistory',
		width: 560,
		height: 800
	};
	nw.Window.open('app/windows/downloadHistory/html/downloadHistory.html', winDownloadHistoryOptions, function(win) {
		_popupWindow = win;
		win.on("closed", function () {
			_popupWindow = null;
		});
	});
}

/* Public */
module.exports = {
	setShowSaveAs: _setShowSaveAs,
	getShowSaveAs: function() {
		return _showSaveAs;
	},
	setDefaultLocation: _setDefaultLocation,
	getDefaultLocation: function() {
		return _defaultLocation;
	},
	getTemporaryDownloadLocation: _getTemporaryDownloadLocation,
	openHistory: _openHistory,
	reloadDownloadSettings: function() {
		_dlSettings = global.settings.downloads;
	},
};
