/**
 * Apply settings from an ini file
 */

const ini = require('ini');

/**
 * Parses an ini file and creates settings from it.
 * @param {String} filename path to the ini file
 */
function importINI(filename) {
	var config;
	try {
		config = ini.parse(global.fs.readFileSync(filename, 'utf-8'));

	} catch (e) {
		console.log(`File ${filename} could not be opened.`);
		return;
	}

	/* Only apply settings from ini if there are none yet. Except forceImport is true */
	if (localStorage.getItem('settings') && !config.forceImport) {
		return;
	}

	var newSettings = {
		'general': {},
		'profiles': {},
		'downloads': {
			'downloadBehavior': 'ask',
		},
		'spellcheck': {},
		'language': {},
	};

	/* Spellcheck section */
	if (config.spellcheck) {
		if (config.spellcheck.languages && config.spellcheck.languages.length > 0) {
			chrome.settingsPrivate.setPref(
				"spellcheck.dictionaries",
				config.spellcheck.languages, "",
				function() {});
			chrome.settingsPrivate.setPref(
				"intl.accept_languages",
				config.spellcheck.languages.join(), "",
				function() {});
		}
	}

	/* Language section */
	if (config.language) {
		if (config.language.defaultUiLang) {
			chrome.settingsPrivate.setPref(
				"intl.app_locale",
				config.language.defaultUiLang, "",
				function() {});
		}
		newSettings.language.defaultUiLang = config.language.defaultUiLang;
	}

	/* general section */
	if (config.general) {
		if (config.general.zoomFactor) {
			config.general.zoomLevel = global.DA.zoomFactorToLevel(config.general.zoomFactor);
		}
		newSettings.general = JSON.parse(JSON.stringify(config.general));
	}

	/* Downloads section */
	if (config.downloads) {
		if (typeof(config.downloads.showSaveAs) !== 'undefined') {
			chrome.settingsPrivate.setPref("download.prompt_for_download",
				config.downloads.showSaveAs, '',
				function() {});
		}
		if (config.downloads.defaultDownloadLocation) {
			chrome.settingsPrivate.setPref("download.default_directory",
				replaceENV(config.downloads.defaultDownloadLocation), '',
				function() {});
		}
		if (config.downloads.hasOwnProperty('openOnCompleted')) {
			newSettings.downloads.downloadBehavior = config.downloads.openOnCompleted ? 'open' : 'ask';
		}
		if (config.downloads.hasOwnProperty('downloadBehavior')) {
			newSettings.downloads.downloadBehavior = config.downloads.downloadBehavior;
		}
	}


	/* Now build the profile objects */
	if (config.profiles) {
		for (let profileId in config.profiles) {
			if (config.profiles.hasOwnProperty(profileId)) {
				let profile = config.profiles[profileId];
				/* 
				 * Some properties allow the usage of ${var}.
				 * They will be evaluted by enclosing them in literal strings.
				 * The purpose of that is to be able to use environment variables.
				 * e.g.: username=${USERNAME}@domain.com
				 */
				['name', 'email', 'url', 'username'].forEach(function(property) {
					profile[property] = profile.hasOwnProperty(property) ?
						replaceENV(profile[property]) : '';
				});
				/* If a password is set, encrypt it */
				profile.password = profile.hasOwnProperty('password') ?
					DAEncryption.encrypt(profile.password) : '';

				profile.authType = profile.hasOwnProperty('authType') ?
					profile.authType : 'webapp';

				profile.rememberPassword = profile.hasOwnProperty('rememberPassword') ?
					profile.rememberPassword : (profile.authType === 'sso' ? false : true);

				newSettings.profiles[profileId] = {
					"id": profileId,
					"name": profile.name,
					"email": profile.email,
					"passwordEncryption": DAEncryption.getVersion(),
					"rememberPassword": profile.rememberPassword,
					"authType": profile.authType,
					"requestOptions": {
						"url": profile.url,
					}
				};
				if (profile.authType !== 'sso') {
					newSettings.profiles[profileId].requestOptions.form = {
						"username": profile.username,
						"password": profile.password
					};
				}
			}
		}
	}
	global.settings = JSON.parse(JSON.stringify(newSettings));
	global.DA.saveSettings();
}


/**
 * This checks the various possibilities where a ini file could be imported from.
 * - commandline argument --importini=<absolute-path-to-ini>
 * - default location nw.App.dataPath/deskapp.ini
 * - path from registry key (windows only)
 */
function checkForINI() {
	var importINIFilename = '';

	/* First check for the commandline argument --importini=<pathtoini> */
	for (let arg of nw.App.argv) {
		if (/^--importini=/.test(arg)) {
			importINIFilename = arg.replace(/^--importini=/g, '');
			break;
		}
	}

	/* Check the data dir for deskapp.ini */
	var filePath = global.path.join(nw.App.dataPath, 'deskapp.ini');
	if (global.fs.existsSync(filePath)) {
		importINIFilename = filePath;
	}

	/*
	 * Check the registry keys on windows
	 * HKLM supersedes HKCU
	 */
	if (/^win/.test(process.platform)) {
		var stdout;
		try {
			stdout = require('child_process').execSync(
				'REG QUERY "HKLM\\Software\\Kopano\\Kopano DeskApp" /v importINI', {
					'encoding': 'utf-8'
				});
		} catch (e) {}
		try {
			stdout = require('child_process').execSync(
				'REG QUERY "HKCU\\Software\\Kopano\\Kopano DeskApp" /v importINI', {
					'encoding': 'utf-8'
				});
		} catch (e) {}

		if (stdout) {
			importINIFilename = stdout.trim().split(/\r?\n/)[1].trim().split(/\s+/)[2];
		}
	}

	if (importINIFilename !== '') {
		importINI(importINIFilename);
	}
}

/**
 * Replaces ${ENVVAR} by its corresponding environment variable.
 * @param {String} str input string
 * @return {String} the resulting string
 */
function replaceENV(str) {
	const regex = /\${([^}]*)}/g;
	let m;

	while ((m = regex.exec(str)) !== null) {
		str = str.replace(m[0], process.env.hasOwnProperty(m[1]) ? process.env[m[1]] : m[1]);
	}

	return str;
}
